<?php
/**
 * The singleton class responsible for registering the custom post types related to the Muut plugin.
 * Currently this is only tied to Webhooks and storing data directly related to those.
 *
 * @package   Muut
 * @copyright 2014 Muut Inc
 */

// Don't load directly
if ( !defined( 'ABSPATH' ) ) {
	die( '-1' );
}

if ( !class_exists( 'Muut_Custom_Post_Types' ) ) {

	/**
	 * Muut Custom Post Types class.
	 *
	 * @package Muut
	 * @author  Paul Hughes
	 * @since   3.0
	 */
	class Muut_Custom_Post_Types
	{
		const MUUT_THREAD_CPT_NAME = 'muut_thread';

		const MUUT_REPLY_TYPE_NAME = 'muut_post_reply';

		const MUUT_PUBLIC_POST_STATUS = 'muut_public';

		const MUUT_SPAM_POST_STATUS = 'muut_spam';

		const MUUT_SPAM_INHERIT_STATUS = 'muut_spam_inherit';


		/**
		 * @static
		 * @property Muut_Custom_Post_Types The instance of the class.
		 */
		protected static $instance;

		/**
		 * The singleton method.
		 *
		 * @return Muut_Custom_Post_Types The instance.
		 * @author Paul Hughes
		 * @since  3.0
		 */
		public static function instance() {
			if ( !is_a( self::$instance, __CLASS__ ) ) {
				self::$instance = new self();
			}
			return self::$instance;
		}

		/**
		 * The class constructor.
		 *
		 * @return Muut_Custom_Post_Types
		 * @author Paul Hughes
		 * @since  3.0
		 */
		protected function __construct() {
			$this->addActions();
			$this->addFilters();
		}

		/**
		 * Adds the actions used by this class.
		 *
		 * @return void
		 * @author Paul Hughes
		 * @since  3.0
		 */
		protected function addActions() {
			add_action( 'init', array( $this, 'registerCustomPostStatuses' ) );
			add_action( 'init', array( $this, 'registerCustomPostTypes' ) );
		}

		/**
		 * Adds the filters used by this class.
		 *
		 * @return void
		 * @author Paul Hughes
		 * @since  3.0
		 */
		protected function addFilters() {

		}

		/**
		 * Register the custom post types.
		 *
		 * @return void
		 * @author Paul Hughes
		 * @since 3.0.2
		 */
		public function registerCustomPostTypes() {
			// Only worrying about a couple of these, since the post type is TOTALLY not public at all.
			$muut_post_labels = array(
				'name' => __( 'Muut Threads', 'muut' ),
				'singular_name' => __( 'Muut Thread', 'muut' ),
			);

			$muut_post_args = apply_filters( 'muut_thread_cpt_args', array(
				'label' => __( 'Muut Threads', 'muut' ),
				'labels' => $muut_post_labels,
				'description' => __( 'This post type is not for public use, but is mainly for storing Muut thread data passed to it via webhooks.', 'muut' ),
				'public' => false,
			) );

			register_post_type( self::MUUT_THREAD_CPT_NAME, $muut_post_args );
		}

		/**
		 * Register the custom post types post statuses.
		 *
		 * @return void
		 * @author Paul Hughes
		 * @since 3.0.2
		 */
		public function registerCustomPostStatuses() {
			$muut_post_status_public_args = apply_filters( 'muut_public_post_status_args', array(
				'label' => __( 'Public Muut Post', 'muut' ),
				'public' => false,
			) );

			$muut_post_status_spam_args = apply_filters( 'muut_spam_post_status_args', array(
				'label' => __( 'Spammed Muut Post', 'muut' ),
				'public' => false,
			) );

			register_post_status( self::MUUT_PUBLIC_POST_STATUS, $muut_post_status_public_args );
			register_post_status( self::MUUT_SPAM_POST_STATUS, $muut_post_status_spam_args );
		}

		/**
		 * Add new Muut Thread.
		 *
		 * @param array $args The post args.
		 *                    This array should follow the following structure:
		 *                    'title' => "The Thread Title"
		 *                    'path' => The new thread path.
		 *                    'user' => The *Muut* username.
		 *                    'body' => The thread content.
		 * @return int The CPT post id.
		 * @author Paul Hughes
		 * @since 3.0.2
		 */
		public function addMuutThreadData( $args = array() ) {
			if ( empty( $args['title'] ) || empty( $args['path'] ) || empty( $args['user'] ) || !isset( $args['body'] ) ) {
				return false;
			}

			// If everything is there, lets add the thread.
			extract( $args );

			$post_args = array(
				'post_title' => $title,
				'post_content' => $body,
				'post_name' => urlencode( $path ),
				'post_type' => self::MUUT_THREAD_CPT_NAME,
				'post_status' => self::MUUT_PUBLIC_POST_STATUS,
			);

			// Add the thread to the Posts table.
			$inserted_post = wp_insert_post( $post_args, false );

			if ( $inserted_post == 0 ) {
				return false;
			}

			// Add the muut user as post meta.
			update_post_meta( $inserted_post, 'muut_user', $user );

			// Add the Muut post path as meta (even though it is also stored in post_name on the posts table).
			update_post_meta( $inserted_post, 'muut_path', $path );

			// Add the Muut channel path.
			$path_split = explode( '#', $path );
			update_post_meta( $inserted_post, 'muut_channel_path', $path_split[0] );

			// Return the WP post id.
			return $inserted_post;
		}

		/**
		 * Add new Muut Reply.
		 *
		 * @param array $args The comment args.
		 *                    This array should follow the following structure:
		 *                    'key' => The Muut thread key
		 *                    'path' => The main thread path.
		 *                    'user' => The *Muut* username.
		 *                    'body' => The reply content.
		 * @return int The comment id.
		 * @author Paul Hughes
		 * @since 3.0.2
		 */
		public function addMuutReplyData( $args = array() ) {
			if ( empty( $args['key'] ) || empty( $args['path'] ) || empty( $args['user'] ) || !isset( $args['body'] ) ) {
				return false;
			}

			$comment_type = self::MUUT_REPLY_TYPE_NAME;

			// If everything is there, lets add the thread.
			extract( $args );

			// Check if a WP post exists in the database that would match the path of the "post" request (for threaded commenting).
			$query_args = array(
				'post_type' => Muut_Custom_Post_Types::MUUT_THREAD_CPT_NAME,
				'post_status' => Muut_Custom_Post_Types::MUUT_PUBLIC_POST_STATUS,
				'meta_query' => array(
					array(
						'key' => 'muut_path',
						'value' => $path,
					),
				),
				'posts_per_page' => 1,
			);

			// Get the post data.
			$posts_query = new WP_Query;
			$posts = $posts_query->query( $query_args );

			$post_id = isset( $posts[0]->ID ) ? $posts[0]->ID : 0;

			// Update the post modified date for the parent thread WP post.
			if ( $post_id > 0 ) {
				wp_update_post( array( 'ID' => $post_id ) );
			}

			$comment_args = array(
				'comment_post_ID' => $post_id,
				'comment_content' => $body,
				'comment_type' => $comment_type,
				'comment_agent' => 'Muut ' . muut()->getMuutVersion() . '; ' . muut()->getForumName(),
			);

			// Add the thread to the Posts table.
			$inserted_reply = wp_insert_comment( $comment_args );

			if ( $inserted_reply == 0 ) {
				return false;
			}

			// Add the muut comment meta.
			update_comment_meta( $inserted_reply, 'muut_user', $user );

			update_comment_meta( $inserted_reply, 'muut_key', $key );

			update_comment_meta( $inserted_reply, 'muut_path', $path );

			// Return the comment id.
			return $inserted_reply;
		}

		/**
		 * Mark a post as spam.
		 *
		 * @param int $post_id The post ID we are marking as spam.
		 * @return int The post ID or 0 if failed.
		 * @author Paul Hughes
		 * @since 3.0.2.1
		 */
		public function markPostAsSpam( $post_id ) {
			if ( !is_numeric( $post_id ) || get_post_type( $post_id ) !== self::MUUT_THREAD_CPT_NAME ) {
				return 0;
			}

			// Mark child comments as spam—inherited (so they are unmarked if this post is unmarked).
			$comment_query_args = array(
				'post_id' => $post_id,
				'status' => 'approve',
			);
			$comment_query = new WP_Comment_Query;
			$comments = $comment_query->query( $comment_query_args );
			foreach( $comments as $comment ) {
				$comment_args = array(
					'comment_ID' => $comment->comment_ID,
					'comment_approved' => self::MUUT_SPAM_INHERIT_STATUS,
				);
				wp_update_comment( $comment_args );
			}

			$post_args = array(
				'ID' => $post_id,
				'post_status' => self::MUUT_SPAM_POST_STATUS,
			);

			// Return the comment id.
			return wp_update_post( $post_args );
		}

		/**
		 * Mark a post as NOT spam (unspam the post).
		 *
		 * @param int $post_id The post ID we are unspamming.
		 * @return int The post ID or 0 if failed.
		 * @author Paul Hughes
		 * @since 3.0.2.1
		 */
		public function markPostAsNotSpam( $post_id ) {
			if ( !is_numeric( $post_id ) || get_post_type( $post_id ) !== self::MUUT_THREAD_CPT_NAME ) {
				return 0;
			}

			// Mark child comments as spam—inherited (so they are unmarked if this post is unmarked).
			$comment_query_args = array(
				'post_id' => $post_id,
				'status' => self::MUUT_SPAM_INHERIT_STATUS,
			);
			$comment_query = new WP_Comment_Query;
			$comments = $comment_query->query( $comment_query_args );
			foreach( $comments as $comment ) {
				$comment_args = array(
					'comment_ID' => $comment->comment_ID,
					'comment_approved' => 1,
				);
				wp_update_comment( $comment_args );
			}

			$post_args = array(
				'ID' => $post_id,
				'post_status' => self::MUUT_PUBLIC_POST_STATUS,
			);

			// Return the comment id.
			return wp_update_post( $post_args );
		}

		/**
		 * Mark a post as spam.
		 *
		 * @param int $post_id The post ID we are marking as spam.
		 * @return int The post ID or 0 if failed.
		 * @author Paul Hughes
		 * @since 3.0.2.1
		 */
		public function markCommentAsSpam( $comment_id ) {
			if ( !is_numeric( $comment_id ) ) {
				return 0;
			}

			$comment_args = array(
				'comment_ID' => $comment_id,
				'comment_approved' => self::MUUT_SPAM_POST_STATUS,
			);

			// Return the comment id.
			return wp_update_comment( $comment_args );
		}

		/**
		 * Mark a post as NOT spam (unspam the post).
		 *
		 * @param int $post_id The post ID we are unspamming.
		 * @return int The post ID or 0 if failed.
		 * @author Paul Hughes
		 * @since 3.0.2.1
		 */
		public function markCommentAsNotSpam( $comment_id ) {
			if ( !is_numeric( $comment_id ) ) {
				return 0;
			}

			$comment_args = array(
				'comment_ID' => $comment_id,
				'comment_approved' => 1,
			);

			// Return the comment id.
			return wp_update_comment( $comment_args );
		}
	}
}